package org.drools.brms.server.contenthandler;
/*
 * Copyright 2005 JBoss Inc
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



import java.io.IOException;

import org.drools.brms.server.builder.BRMSPackageBuilder;
import org.drools.brms.server.builder.ContentPackageAssembler;
import org.drools.compiler.DroolsParserException;
import org.drools.repository.AssetItem;

/**
 * This inferface indicates that an asset is a rule asset content type, 
 * ie not a model, dsl etc that supports package compilation.
 * 
 * @author Michael Neale
 */
public interface IRuleAsset {

    /**
     * This will be called when the asset is required to compile itself, 
     * in the context of the given builder.
     */
    public void compile(BRMSPackageBuilder builder, AssetItem asset, ContentPackageAssembler.ErrorLogger logger) throws DroolsParserException,
                                                                    IOException;

    /**
     * This will be called when a rule asset is to render itself to DRL source.
     */
    public void assembleDRL(BRMSPackageBuilder builder, AssetItem asset, StringBuffer buf);
    
}